/***************************************************************************\
  "error.h" - Error Handling Functions (Global Phase)
  Revision 16; 10 Nov 2002 - 21 Feb 2004
  Copyright (C) 2002-2004  Kevin P. Albrecht
\***************************************************************************/

#ifndef ERROR_H
#define ERROR_H

/////////////////////////
// DEFINES //////////////
/////////////////////////

// External Defines //

// "Program" Error Codes (Controller) //
#define EP_INVALID_ARG               "P01: Invalid command line argument"
#define EP_INVALID_ARGS              "P02: Invalid command line arguments"
#define EP_MISSING_ARG               "P03: Missing command line argument"
#define EP_FILE_OPEN_FAILED          "P04: Error opening file"

// Error Codes for "Front-End" (Phases A, B, and C) //
#define EF_UNDEFINED_TOKEN            "F01: Undefined token"
#define EF_UNDECLARED_IDENTIFIER      "F02: Undeclared identifier"
#define EF_UNEXPECTED_TOKEN           "F03: Unexpected token"
#define EF_UNEXPECTED_EOF             "F04: Unexpected end-of-file"
//#define EF_UNNECESSARY_TYPE_CONV      "F05: Unnecessary type conversion"
#define EF_MISSING_EOL                "F06: Missing end-of-line"
//#define EF_MISSING_ASSIGNMENT_OP      "F07: Missing assignment operator"
//#define EF_MISSING_RIGHT_PAREN        "F08: Missing right parenthesis"
#define EF_INVALID_EXPRESSION         "F09: Invalid expression"
#define EF_ALREADY_DECLARED           "F10: Identifier already declared"
#define EF_TYPE_INCOMPATIBLE          "F11: Type is incompatible"
#define EF_INVALID_PARSE_POINT        "F12a: Invalid parsing point"
#define EF_INCOMPLETE_STR_LIT         "F12b: String is never closed"
//#define EF_INCOMPLETE_COMMENT         "F13: Comment is never closed"
//#define EF_CONST_ASSIGNMENT           "F14: Cannot assign a new value to a constant"
//#define EF_MAX_SCOPES_REACHED         "F15: Maximum concurrent scopes reached"
//#define EF_MIN_SCOPES_REACHED         "F16a: Minimum scopes reached; Cannot leave global scope"
//#define EF_INDENT_ERROR               "F16b: Decreased indent must match indent of outer scope"
//#define EF_MISSING_TYPE_DEF_OP        "F17: Missing type definition operator"
//#define EF_INVALID_WHITESPACE         "F18: Invalid whitespace character"
//#define EF_NOT_VALID_TYPE             "F19: Not a valid type"
//#define EF_MISSING_EXPRESSION         "F20: Missing expression"
//#define EF_ROUT_REDEF                 "F21: Redefinition of routine"
//#define EF_RET_OUTSIDE_OF_ROUT        "F22: Return outside of routine"
//#define EF_NON_MATCHING_RET_TYPE      "F23: Returned type does not match routine"
//#define EF_MAIN_NOT_DEF               "F24: 'main' routine not defined"
//#define EF_RETURN_NEVER_MADE          "F25: No return from routine"
//#define EF_MAIN_NOT_VOID              "F26: 'main' return type should be void, not"
//#define EF_NO_RETURN_FOR_VOID_ROUT    "F27: Cannot return a value when routine has no type"
//#define EF_EMPTY_BLOCK                "F28: Empty block encountered"
//#define EF_BAD_STATEMENT_OUTSIDE_ROUT "F29: Statement illegal outside of routine"
//#define EF_SYM_READDED_TO_LOCVAR_LIST "FX30: Symbol can only be added to routine's list once"
//#define EF_CANNOT_CLONE_A_ROUTINE     "FX31: Cannot clone a routine"
//#define EF_NO_PARAMETER_FOR_ARGUMENT  "F32: Routine call passed too many arguments"
//#define EF_ARG_TYPE_MISMATCH          "F33: Argument's type does not match routine definition"
//#define EF_ARG_NUM_MISMATCH           "F34: Argument number does not match routine definition"
//#define EF_NO_RETURN_VALUE            "F35: Routine does not return a value"
//#define EF_RETURN_VAL_NOT_ASSIGNED    "F36: Value returned from routine must be used"
//#define EF_MISSING_IDENTIFIER         "F37: Missing identifier"

// Error Codes for "Intermediate Code Manager" (Phase D) //
#define EI_WRITE_AFTER_RESET         "I01: Intermediate code write attempt after reset"
#define EI_READ_BEFORE_RESET         "I02: Intermediate code read attempt before reset"
#define EI_GOTO_BEFORE_RESET         "I03: Intermediate code goto attempt before reset"
#define EI_CURR_INDEX_OUT_OF_BOUNDS  "I04: Current intermediate code index out of bounds"
//#define EI_MAX_TEMPS_EXCEEDED        "I05: Maximum number of temporary variables exceeded"
//#define EI_MIN_TEMPS_REACHED         "I06: Minimum number of temporary variables reached"
//#define EI_MAX_UNIQUE_NAMES_EXCEEDED "I07: Maximum number of unique names exceeded"

// Error Codes for "Runtime Engine" (Phase E) //
#define ER_INVALID_INSTRUCTION       "R01: Invalid instruction encountered"
#define ER_LABEL_NOT_DEFINED         "R02: Jump was not defined by a label"
#define ER_ROUTINE_LABEL_NOT_DEFINED "R02a: Call was not defined by a label"
#define ER_DIV_BY_ZERO               "R03: Divide by zero attempted"
#define ER_ASCII_ZERO                "R03: Cannot creae ASCII code from zero"
#define ER_POP_MISMATCHED_TYPE       "R04: Destination of pop must match type on stack"
#define ER_MISMATCHED_TYPES          "R04: Items do not match type expected by operation"
//#define ER_POP_STACK_EMPTY           "R05: Attempted to pop from empty stack"
//#define ER_RET_POP_NOT_LABEL_TYPE    "R04: Return attempt failed; Type on stack not label"
#define ER_STACK_EMPTY               "R05: Stack access failed; Stack empty"
#define ER_END_OF_FILE               "R06: End of file prematurely reached"
#define ER_MAX_FILES_OPENED          "R07: Too many files opened"
#define ER_FILE_OPEN_ERROR           "R08: File cannot be opened"
#define ER_INVALID_FILE_HANDLE       "R09: File handle is not valid"
#define ER_INVALID_FILE_MODE         "R10: Attempted to read from output (or the reverse)"
#define ER_FIO_R_EOF_REACHED         "R11: End of file reached"

// Effect Codes //
#define EFF_IGNORE 0
#define EFF_RECOVER 1
#define EFF_TERMINATE 2

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //
#include <stdio.h>
#include <stdlib.h>

// Local //
#include "global.h"
#include "k_string.h"

/////////////////////////
// DECLARATIONS /////////
/////////////////////////

void err (bool lines, char *sError, char *sClarification);
void errm (char *s1, char *s2);
void erre (int nEffect);

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

//// err
//// Display an error
void err (bool lines, char *sError, char *sClarification)
{
  gnErrors++;

  fprintf (stderr, "** Error ");

  if (lines)
  {
    if (gbDebugErrorsOn)
    {
      fprintf (stderr, "(%s:%d) ", gsDebugFilename, gnDebugLine);
    }
    else
    {
      fprintf (stderr, "(%s:%d) ", gsCurrentFilename, gnCurrentLine);
    }
  }

  fprintf (stderr, "%s", sError);
  if (sClarification) fprintf (stderr, ": %s", sClarification);
  fprintf (stderr, "\n");
}

//// errm
//// Display an additional error message
void errm (char *s1, char *s2)
{
  if (s1) fprintf (stderr, "**   (continued...) %s", s1);
  if (s2) fprintf (stderr, ": %s", s2);
  fprintf (stderr, "\n");
}

//// erre
//// Display and/or execute an error effect
void erre (int nEffect)
{
  switch (nEffect)
  {
    case (EFF_IGNORE):
      break;
    case (EFF_RECOVER):
      break;
    case (EFF_TERMINATE):
      fprintf (stderr, "** Fatal Error: Terminating program...\n");
      exit (0);
      break;
    default:
      fprintf (stderr, "** Invalid Error Effect: %d\n", nEffect);
      break;
  }
}

#endif
