/***************************************************************************\
  "k_conv.h" - Kevin's Type Conversion Functions
  Revision 7; 10 Nov 2002 - 14 March 2004
  Available at: "http://kevin.alteu.com/"
  E-mail:       "kevin@albrecht.net"

  Copyright (C) 2002-2004, Kevin P. Albrecht

  Permission is hereby granted by Kevin P. Albrecht (the "Author"), free of
  charge, to any person obtaining a copy of this software and associated
  documentation files (the "Software"), to deal in the Software without
  restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the
  Software, and to permit persons to whom the Software is furnished to do
  so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included
  in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
  THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
  AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

  Except as contained in this notice, the name of the Author shall not be
  used in advertising or otherwise to promote the sale, use or other
  dealings in this Software without prior written authorization from the
  Author.
\***************************************************************************/

#ifndef K_CONV_H
#define K_CONV_H

/////////////////////////
// DEFINES //////////////
/////////////////////////

// External Defines //
// GLOBAL_MANTISSA_SIZE

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //

// Local //
#include "k_string.h"
#include "k_track.h"

/////////////////////////
// DECLARATIONS /////////
/////////////////////////

char *k_conv_bs (bool ab);
char *k_conv_is (int an);
char *k_conv_ds (double ad);
char *k_conv_cs (char ac);
bool k_conv_sb (char *aS);
bool k_conv_sbD (char *aS);
int k_conv_si (char *aS);
int k_conv_siD (char *aS);
double k_conv_sd (char *aS);
double k_conv_sdD (char *aS);

double *k_new_d (double d);
bool *k_new_b (bool b);
int *k_new_n (int n);

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

char *k_conv_bs (bool ab)
{
  if (ab) return "true";
  else    return "false";
}

char *k_conv_is (int an)
{
  return k_int (an);
}

char *k_conv_ds (double ad)
{
  return k_double (ad, GLOBAL_MANTISSA_SIZE);
}

char *k_conv_cs (char ac)
{
  char *s = k_new (2);
  s[0] = ac;
  s[1] = '\0';
  return s;
}

bool k_conv_sb (char *aS)
{
  if (aS == NULL)
  {
    return false;
  }
  else if (k_same (aS, "false"))
  {
    return false;
  }
  else if (k_same (aS, "true"))
  {
    return true;
  }
  else if (k_conv_sd (aS) == 0.0)
  {
    return false;
  }
  else
  {
    return true;
  }
}

bool k_conv_sbD (char *aS)
{
  bool temp = k_conv_sb (aS);
  k_del (aS);
  return temp;
}

int k_conv_si (char *aS)
{
  if (aS == NULL) return 0;
  return (atoi (aS));
}

int k_conv_siD (char *aS)
{
  int temp = k_conv_si (aS);
  k_del (aS);
  return temp;
}

double k_conv_sd (char *aS)
{
  if (aS == NULL) return 0.0;
  return (atof (aS));
}

double conv_sdD (char *aS)
{
  double temp = k_conv_sd (aS);
  k_del (aS);
  return temp;
}

double *k_new_d (double d)
{
  double *pd;
  pd = (double *) KALLOC(sizeof(double));
  *pd = d;
  return pd;
}

bool *k_new_b (bool b)
{
  bool *pb;
  pb = (bool *) KALLOC(sizeof(bool));
  *pb = b;
  return pb;
}

int *k_new_n (int n)
{
  int *pn;
  pn = (int *) KALLOC(sizeof (int));
  *pn = n;
  return pn;
}

#endif
