/***************************************************************************\
  "token.h" - Token Variables and Functions (Phase B)
  Revision 21; 10 Nov 2002 - 26 Feb 2004
  Copyright (C) 2002-2004  Kevin P. Albrecht
\***************************************************************************/

#ifndef TOKEN_H
#define TOKEN_H

/////////////////////////
// DEFINES //////////////
/////////////////////////

// External Defines //
// GLOBAL_TOKEN_SIZE

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //

// Local //
#include "global.h"

/////////////////////////
// DECLARATIONS /////////
/////////////////////////

typedef enum
{
  // null //
  ttNull, ttError,

  // ttStart is passed to lex_get at the start of the file //
  ttStart,

  // valued token types //
  ttIdentifier, ttIntLiteral, ttRatLiteral, ttStrLiteral,
  ttBoolLiteral,

  // source tt whitespace types //
  ttEOF, ttEOL,

  // special characters //
  ttColon,

  // reserved words //
  ttNOP, ttCALL, ttRET, ttSTOP,
  ttJMP, ttJT, ttJF,
  ttEQ, ttNEQ, ttLT, ttGT, ttLTEQ, ttGTEQ,
  ttOR, ttAND, ttNOT,
  ttDB, ttDI, ttDR, ttDS,
  ttTOB, ttTOI, ttTOS, ttTOR,
  ttCIN, ttCOUT,
  ttFIO_OR, ttFIO_OW, ttFIO_C, ttFIO_R, ttFIO_W, ttFIO_EOF,
  ttRAND, ttASC, ttNEG, ttCAT, 
  ttADDI, ttSUBI, ttMULI, ttDIVI, ttPOWI, ttMODI, 
  ttADDR, ttSUBR, ttMULR, ttDIVR, ttPOWR, ttMODR, 
  ttCOPY, ttSWAP,
  ttPOP, ttPOPL, ttPOPP,
  ttPUSH, ttPUSHL, ttPUSHP, 
  ttDBGON, ttDBGLINE, ttDBGFILE

} TokenCode;

char *gvsTokenCodeStrings[] =
{
  // null //
  "<null>", "<error>",

  // ttStart is passed to lex_get at the start of the file //    
  "<start>",

  // valued token types //
  "<identifier>", "<int_literal>", "<rat_literal>", "<str_literal>", 
  "<bool_literal>",

  // source code whitespace types //
  "<eof>", "<eol>",

  // special characters //
  ":",

  // reserved words //
  "nop", "call", "ret", "stop",
  "jmp", "jt", "jf",
  "eq", "neq", "lt", "gt", "lteq", "gteq",
  "or", "and", "not",
  "db", "di", "dr", "ds",
  "tob", "toi", "tos", "tor",
  "cin", "cout",
  "fio_or", "fio_ow", "fio_c", "fio_r", "fio_w", "fio_eof",
  "rand", "asc", "neg", "cat",
  "addi", "subi", "muli", "divi", "powi", "modi", 
  "addr", "subr", "mulr", "divr", "powr", "modr", 
  "copy", "swap",
  "pop", "popl", "popp",
  "push", "pushl", "pushp",
  "dbgon", "dbgline", "dbgfile"
};

typedef struct
{
  TokenCode code;
  char lexeme[GLOBAL_TOKEN_SIZE]; // the actual text of the token as it appears in input
  void *value; // the value held in the token:
               // CODE            VALUE          MEANING
               // ----            -----          -------
               // ttIntLiteral    int*           integer
               // ttRatLiteral    float*         rational
               // ttStrLiteral    char*          string
               // ttBoolLiteral   bool*          boolean
               // (other)         void*          (none)
} Token;

void tok_clear (Token *token);
Token *tok_new ();
void tok_del (Token *token);
void tok_set (Token *token, void *aValue);

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

void tok_clear (Token *token)
{
  token->code = ttNull;
  k_copy (token->lexeme, "");
  token->value = NULL;
}

void tok_del (Token *token)
{
  // token may be NULL //
  if (!token) return;

  if (token->code == ttStrLiteral)
  {
    if (((char*)(token->value))[0] != 0) // this is a bug fix so that deleting an empty string won't cause an error.
      KFREE(token->value);
  }
  else
  {
    KFREE(token->value);
  }

  KFREE(token);
}

Token *tok_new ()
{
  Token *temp = (Token *) KALLOC(sizeof (Token));
  tok_clear (temp);
  return temp;
}

void tok_set (Token *token, void *aValue)
{
  token->value = aValue;
}

#endif
